from manifest_edit.plugin.mpd import ManifestIteratorPlugin
from schema import Schema, Or
import re


class Plugin(ManifestIteratorPlugin):
    '''
    essential_property_remove plugin

    This plugin will remove one or more EssentialProperty elements from a
    specified position in a manifest.

    It can be used to remove all EssentialProperty elements, or only those with
    a specific scheme or value. Schemes and values are checked using regular
    expressions.

    For example, to remove all from any Adaptation Set:

    .. code-block:: yaml

        periods:
        - '*': '.*'
          adaptationSets:
          - '*': '.*'
            essentialPropertyScheme: '.*'
            essentialPropertyValue: '.*'

    Or to only remove a specific property:

    .. code-block:: yaml

        periods:
        - '*': '.*'
          adaptationSets:
          - '*': '.*'
            essentialPropertyScheme: 'urn:dvb:dash:lowlatency:critical:2019'
            essentialPropertyValue: 'true'

    '''

    _name = __name__

    _keys = ['essentialPropertyScheme', 'essentialPropertyValue']

    def schema(self):
        return Schema({
            self._keys[0]: str,
            self._keys[1]: Or(str, int)
        })

    def removeEssentialProperties(self, manifest, storage):
        for ess_prop_config, element in self.config(manifest, storage):
            properties_to_remove = []

            for prop in element.essentialProperties:
                if (re.fullmatch(ess_prop_config['essentialPropertyScheme'],
                                 prop.schemeIdUri)
                    and
                    re.fullmatch(ess_prop_config['essentialPropertyValue'],
                                 prop.value)):
                    properties_to_remove.append(prop)

            for prop in properties_to_remove:
                element.essentialProperties.remove(prop)

    def process(self, manifest, storage):
        self.removeEssentialProperties(manifest, storage)
